package ts.test.entity;

import java.io.Serializable;
import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.Lob;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;
import javax.validation.constraints.NotNull;

import org.hibernate.validator.constraints.NotBlank;

@Entity
@Table(name = "blah")
@NamedQueries({
	@NamedQuery(name = "Blah.findById", query = "SELECT b FROM Blah b WHERE b.id = :id"), 
    @NamedQuery(name = "Blah.getAll", query = "SELECT b FROM Blah b ORDER BY b.name ASC")
})
public class Blah implements Serializable, Comparable<Blah> {

	@Transient
	private static final long serialVersionUID = 1L;
	
	
	@Id
	@NotNull
	@GeneratedValue
	private Long id;
	
	@NotNull
	// NotBlank also ignores whitespaces as far as i can tell
	@NotBlank
	private String name;
	
	@NotNull
	@Temporal(TemporalType.DATE)
	// WORKAROUND: end is a keyword in postgresql and isn't properly escaped by hibernate
	@Column(name="\"end\"")
	private Date end;
	
	// TEXT might be postgresql-specific
	@Lob
	//@Column(columnDefinition="TEXT")
	private String/*Builder*/ description;

	
	public Blah() {
	}
	
	
	public Blah (String name, Date end) {
		super();
		this.name = name;
		this.end = end;
	}
	
	
	public Blah(String name, Date start, String description) {
		this(name, start);
		this.description = description;
	}

	
	public Blah(String name, Date start, Date end) {
		this(name, start);
		this.end = end;
	}

	
	public Blah(String name, Date start, Date end, String description) {
		this(name, start, end);
		this.description = description;
	}


	@Override
	public int compareTo(Blah blah) {
		//return this.name.compareTo(blah.name);
		return this.end.compareTo(blah.end);
	}

	
	// TODO: DANGEROUS !!!
	@Override
	public boolean equals(Object o) {
		return (o instanceof Blah && ((Blah)o).name.equals(this.name) && ((Blah)o).end.equals(this.end));
	}

	
	@Override
	public int hashCode() {
		return name.hashCode() ^ end.hashCode();
		// TODO: Think about this again
		//return name.hashCode();
	}


	public String getName() {
		return name;
	}


	public void setName(String name) {
		this.name = name;
	}


	public Date getEnd() {
		return end;
	}


	public void setEnd(Date end) {
		this.end = end;
	}


	public String getDescription() {
		return description;
	}


	public void setDescription(String description) {
		this.description = description;
	}


	public Long getId() {
		return id;
	}

	
}
